package repository

import (
	"git.chillcy.com/golang/colorway/internal/pkg/database/model"
	"gorm.io/gorm"
	"gorm.io/gorm/clause"
)

type CompleteSpuInterface interface {
	BaseInterface
	UpsertCompleteSpus(completeSpus model.CompleteSpus) error
}

type CompleteSpu struct {
	*Base[model.CompleteSpu]
}

func NewCompleteSpu(db *gorm.DB) *CompleteSpu {
	return &CompleteSpu{
		Base: NewBase[model.CompleteSpu](db),
	}
}

func (receiver *CompleteSpu) UpsertCompleteSpus(completeSpus model.CompleteSpus) error {
	return receiver.db.Transaction(func(db *gorm.DB) error {
		return upsertCompleteSpus(db, completeSpus)
	})
}

func upsertCompleteSpus(db *gorm.DB, completeSpus model.CompleteSpus) error {
	// upsert completeSpus
	result := db.Clauses(clause.OnConflict{
		Columns:   []clause.Column{{Name: "style_number"}},
		DoUpdates: clause.AssignmentColumns([]string{"style_number", "title_cn", "title_en", "description", "season", "brand_id", "care", "made_in", "hs_code", "info_select"}),
	}).Create(&completeSpus)
	if result.Error != nil {
		return result.Error
	}

	for _, spu := range completeSpus {
		err := upsertCompleteSpu(db, spu)
		if err != nil {
			return err
		}
	}

	return nil
}

func upsertCompleteSpu(db *gorm.DB, completeSpu model.CompleteSpu) error {
	// 处理关联关系
	err := db.Model(&completeSpu).Association("ColorSpus").Replace(completeSpu.ColorSpus)
	if err != nil {
		return err
	}
	for _, colorSpu := range completeSpu.ColorSpus {
		err = db.Model(&colorSpu).Association("Skus").Replace(colorSpu.Skus)
		if err != nil {
			return err
		}
		err = db.Model(&colorSpu).Association("Images").Replace(colorSpu.Images)
		if err != nil {
			return err
		}
	}
	err = db.Model(&completeSpu).Association("Compositions").Replace(completeSpu.Compositions)
	if err != nil {
		return err
	}
	err = db.Model(&completeSpu).Association("Models").Replace(completeSpu.Models)
	if err != nil {
		return err
	}
	err = db.Model(&completeSpu).Association("Fits").Replace(completeSpu.Fits)
	if err != nil {
		return err
	}
	err = db.Model(&completeSpu).Association("Genders").Replace(completeSpu.Genders)
	if err != nil {
		return err
	}
	err = db.Model(&completeSpu).Association("Categories").Replace(completeSpu.Categories)
	if err != nil {
		return err
	}

	return nil
}
